#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

#include <plugin/plugin.h>

static FILE*	fdbg	= 0;
static int 	_pid	= 0;

#if defined(DEBUG)
const char* 
pszMessageCode(unsigned int code) {
	switch(code & MSG_BASE_MASK) {
	case MSG_BASE_APPL:
		switch(code) {
		case APPLMSG_Destroy:			return "APPL:APPLMSG_Destroy";
		case APPLMSG_ALARM_DB_Changed:		return "APPL:APPLMSG_ALARM_DB_Changed";
		case APPLMSG_ALARM_Set_Fail:		return "APPL:APPLMSG_ALARM_Set_Fail";
		case APPLMSG_EPG_Data_Ack:		return "APPL:APPLMSG_EPG_Data_Ack";
		case APPLMSG_SI_PAT:			return "APPL:APPLMSG_SI_PAT";
		case APPLMSG_SI_PMT:			return "APPL:APPLMSG_SI_PMT";
		case APPLMSG_SI_NIT:			return "APPL:APPLMSG_SI_NIT";
		case APPLMSG_SI_NIT_Other:		return "APPL:APPLMSG_SI_NIT_Other";
		case APPLMSG_SI_SDT:			return "APPL:APPLMSG_SI_SDT";
		case APPLMSG_SI_SDT_Other:		return "APPL:APPLMSG_SI_SDT_Other";
		case APPLMSG_SI_SDT_Repeat:		return "APPL:APPLMSG_SI_Repeat";
		case APPLMSG_SI_TDT:			return "APPL:APPLMSG_SI_TDT";
		case APPLMSG_SI_TOT:			return "APPL:APPLMSG_SI_TOT";
		case APPLMSG_SI_BAT:			return "APPL:APPLMSG_SI_BAT";
		case APPLMSG_SI_CAT:			return "APPL:APPLMSG_SI_CAT";
		case APPLMSG_TUNER_LOCK(0):		return "APPL:APPLMSG_TUNER0_LOCK";
		case APPLMSG_TUNER_LOCK(1):		return "APPL:APPLMSG_TUNER1_LOCK";
		case APPLMSG_TUNER_LOCK(2):		return "APPL:APPLMSG_TUNER2_LOCK";
		case APPLMSG_TUNER_LOCK(3):		return "APPL:APPLMSG_TUNER3_LOCK";
		case APPLMSG_TUNER_UNLOCK(0):		return "APPL:APPLMSG_TUNER0_UNLOCK";
		case APPLMSG_TUNER_UNLOCK(1):		return "APPL:APPLMSG_TUNER1_UNLOCK";
		case APPLMSG_TUNER_UNLOCK(2):		return "APPL:APPLMSG_TUNER2_UNLOCK";
		case APPLMSG_TUNER_UNLOCK(3):		return "APPL:APPLMSG_TUNER3_UNLOCK";
		case APPLMSG_TUNER_SAME(0):		return "APPL:APPLMSG_TUNER0_SAME";
		case APPLMSG_TUNER_SAME(1):		return "APPL:APPLMSG_TUNER1_SAME";
		case APPLMSG_TUNER_SAME(2):		return "APPL:APPLMSG_TUNER2_SAME";
		case APPLMSG_TUNER_SAME(3):		return "APPL:APPLMSG_TUNER3_SAME";
		case APPLMSG_PluginSelectedCmd:		return "APPL:APPLMSG_PluginSelectedCmd";
		case APPLMSG_PluginReloadCmd:		return "APPL:APPLMSG_PluginReloadCmd";
		case APPLMSG_PluginStart:		return "APPL:APPLMSG_PluginStart";
		case APPLMSG_PluginStop:		return "APPL:APPLMSG_PluginStop";
		case PLUGINMSG_StateChangeStart:	return "APPL:PLUGINMSG_StateChangeStart";
		case PLUGINMSG_StateChanged:		return "APPL:PLUGINMSG_StateChanged";
		case PLUGINMSG_SVC_Starting:		return "APPL:PLUGINMSG_SVC_Starting";
		default:				
			break;
		}
		break;
	case MSG_BASE_CI:
		return "CI:";
		break;
	case MSG_BASE_FILTER:
		return "CI:";
		break;
	case MSG_BASE_KEY:
		return "KEY:";
		break;
	case MSG_BASE_REMOCON:
		return "REMOCON:";
		break;
	case MSG_BASE_TIMER:
		return "TIMER:";
		break;
	case MSG_BASE_TUNER:
		return "TUNER:";
		break;
	}
	return "unknown";
}
#endif

void PluginClose()
{
	system("touch /tmp/gbox.kill");
	WiTask_Sleep(2000);
	system("killall -9 gbox.sh4");
	WiTask_Sleep(100);

#if defined(DEBUG)
	if (fdbg) fprintf(fdbg,"\nexit...\t\n");
	if (fdbg) fclose(fdbg);
#endif
	WiTask_Sleep(200);
	
	WiFront_Term();
}

int PluginMain( int argc, char *argv[] )
{
	int key, mainState = STATE_None;
	dword evt[4] = {0,};

#if defined(DEBUG)
	fdbg = fopen("/tmp/debug.txt","at");
	if (fdbg) fprintf(fdbg,"gbox starter v0.1\n");
#endif

	WiFront_Init();

	mainState = GetState(STATEIDX_Main);

#if defined(DEBUG)
	if (fdbg) fprintf(fdbg,"\ncurrent state %d\t",mainState);
#endif
	
	switch(mainState) {
	case STATE_None:
	default:
		break;
	}

	_pid = getpid();

	system("rm -f /tmp/gbox.kill");
	system("rm -f /tmp/gbox.ver");
	system("rm -f /tmp/share.onl");
	system("rm -f /tmp/share.log");
	system("rm -f /tmp/share.info");
	system("rm -f /tmp/share.stat");
	system("rm -f /tmp/sc*.info");
	system("rm -f /tmp/pid.info");
	
	system("/var/emu/gbox.sh4 & > /dev/null 2>&1");

	while( 1 )
	{
		if( WiQueue_Receive(WiMainQ, (u_int8 *)evt, 16, -1) != 0 ) continue;
		
		/* filter unused messages */
		switch(evt[0]&MSG_BASE_MASK) {
		case MSG_BASE_APPL:
			switch(evt[0]&APPLMSG_BASE_MASK) {
			case APPLMSG_ETC_BASE:
				break;
			default:
				continue;
				break;
			}
			break;
		default:
			break;
		}
		
#if defined(DEBUG)
		if (fdbg) fprintf(fdbg,"%s(%08X)-%08X-%08X-%08X\n",pszMessageCode(evt[0]),evt[0],evt[1],evt[2],evt[3]);
#endif
		///////////////////////////////////////////////////////////////
		
		switch(evt[0]) {
		case PLUGINMSG_StateChangeStart:
			break;
		case PLUGINMSG_StateChanged:
			break;
		case APPLMSG_Destroy:
			// when alarm confirm box open or firmware download box open.. and so on.
			break;
		case APPLMSG_PluginSelectedCmd:
			// press ok on started plugin item in plugin menu.
			if( evt[1] == _pid ) {
			}
			break;
		}
	}


}
